'/*
'Axiom Game Engine Library
'Copyright (C) 2003  Axiom Project Team

'The overall design, and a majority of the core engine and rendering code 
'contained within this library is a derivative of the open source Object Oriented 
'Graphics Engine OGRE, which can be found at http://ogre.sourceforge.net.  
'Many thanks to the OGRE team for maintaining such a high quality project.

'This library is free software; you can redistribute it and/or
'modify it under the terms of the GNU Lesser General Public
'License as published by the Free Software Foundation; either
'version 2.1 of the License, or (at your option) any later version.

'This library is distributed in the hope that it will be useful,
'but WITHOUT ANY WARRANTY; without even the implied warranty of
'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
'Lesser General Public License for more details.

'You should have received a copy of the GNU Lesser General Public
'License along with this library; if not, write to the Free Software
'Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
'*/

Imports System
'Imports Axiom.Collections
'Imports Axiom.Core


'Namespace Animation
' <summary>
'		An animation sequence. 
' </summary>
' <remarks>
'		This class defines the interface for a sequence of animation, whether that
'		be animation of a mesh, a path along a spline, or possibly more than one
'		type of animation in one. An animation is made up of many 'tracks', which are
'		the more specific types of animation.
'		<p/>
'		You should not create these animations directly. They will be created via a parent
'		object which owns the animation, e.g. Skeleton, SceneManager, etc.
' </remarks>
Public Class AnimationNew
#Region " Member variables"

    ' <summary>Name of this animation.</summary>
    Protected mstrName As String
    ' <summary>The total length of this animation (sum of the tracks).</summary>
    Protected msnglength As Single
    ' <summary>Collection of AnimationTracks.</summary>
    Protected trackList As AnimationTrackCollection
    ' <summary>Interpolation mode of this animation.</summary>
    Protected mintInterpolationMode As InterpolationMode
    ' <summary>Default interpolation mode of any animations.</summary>
    Protected Shared mintDefaultInterpolationMode As InterpolationMode

#End Region

#Region " Constructors"

    ' <summary>Static constructor.</summary>
    Shared Sub New()
        ' set default interpolation mode to Spline (mmm....spline)
        DefaultInterpolationMode = InterpolationMode.Spline
    End Sub

    ' <summary>
    '		Internal constructor, to prevent from using new outside of the engine.
    '		<p/>
    '		Animations should be created within objects that can own them (skeletons, scene managers, etc).
    ' </summary>
    Friend Sub New(ByVal name As String, ByVal length As Single)
        Me.mstrName = name
        Me.msnglength = length

        ' use the default interpolation mode
        Me.interpolationMode = AnimationNew.defaultInterpolationMode

        ' initialize the collection
        Me.trackList = New AnimationTrackCollection()
    End Sub


#End Region

#Region " Properties"

    ' <summary>
    '		Gets the name of this animation.
    ' </summary>
    Public ReadOnly Property Name() As String
        Get
            Return mstrName
        End Get
    End Property

    ' <summary>
    '		Gets the total length of this animation.
    ' </summary>
    Public ReadOnly Property Length() As Single
        Get
            Return msngLength
        End Get
    End Property

    ' <summary>
    '		Gets/Sets the current interpolation mode for this animation.
    ' </summary>
    Public Property InterpolationMode() As InterpolationMode
        Get
            Return mintInterpolationMode
        End Get
        Set(ByVal Value As InterpolationMode)
            mintInterpolationMode = Value
        End Set
    End Property

    ' <summary>
    '		A collection of the tracks in this animation.
    ' </summary>
    '// TODO: See if we can ensure that the track list is not modified somehow.
    Public ReadOnly Property Tracks() As AnimationTrackCollection
        Get
            Return trackList
        End Get
    End Property

    ' <summary>
    '		Gets/Sets the default interpolation mode to be used for all animations.
    ' </summary>
    Public Shared Property DefaultInterpolationMode() As InterpolationMode
        Get
            Return mintDefaultInterpolationMode
        End Get
        Set(ByVal Value As InterpolationMode)
            mintdefaultInterpolationMode = Value
        End Set
    End Property

#End Region

#Region " Public methods"

    ' <summary>
    '		Creates an AnimationTrack. 
    ' </summary>
    ' <param name="handle">Numeric handle to give the track, used for accessing the track later.</param>
    ' <returns></returns>
    Public Function CreateTrack(ByVal handle As Short) As AnimationTrack
        Dim track As AnimationTrack = New AnimationTrack(Me)
        track.Handle = handle

        ' add the track to the list
        trackList.Add(track)

        Return track
    End Function

    ' <summary>
    '		Creates a new AnimationTrack automatically associated with a Node. 
    ' </summary>
    ' <param name="index">Numeric handle to give the track, used for accessing the track later.</param>
    ' <param name="target">Node object which will be affected by this track.</param>
    ' <returns></returns>
    'Public Function CreateTrack(ByVal handle As Short, ByVal target As Node) As AnimationTrack
    '    ' create a new track and set it's target
    '    Dim track As AnimationTrack = CreateTrack(handle)
    '    track.TargetNode = target
    '    track.Handle = handle

    '    Return track
    'End Function

    ' <summary>
    '		Applies an animation given a specific time point and weight.
    ' </summary>
    ' <remarks>
    '		Where you have associated animation tracks with Node objects, you can eaily apply
    '		an animation to those nodes by calling this method.
    ' </remarks>
    ' <param name="time">The time position in the animation to apply.</param>
    ' <param name="weight">The influence to give to this track, 1.0 for full influence, less to blend with
    '		other animations.</param>
    ' <param name="accumulate"></param>
    Public Sub Apply(ByVal time As Single, ByVal weight As Single, ByVal accumulate As Boolean)
        ' loop through tracks and update them all with current time
        Dim i As Integer
        For i = 0 To trackList.Count - 1 Step i + 1
            trackList(i).Apply(time, weight, accumulate)
        Next
    End Sub

    'Public Sub Apply(ByVal skeleton As Skeleton, ByVal time As Single, ByVal weight As Single, ByVal accumulate As Boolean)
    '    ' loop through tracks and update them all with current time
    '    Dim i As Integer
    '    For i = 0 To trackList.Count - 1 Step i + 1
    '        Dim track As AnimationTrack = trackList(i)
    '        Dim bone As Bone = skeleton.GetBone(CType(track.Handle, System.UInt16))
    '        track.ApplyToNode(bone, time, weight, accumulate)
    '    Next
    'End Sub

#End Region

#Region " Event handlers"

    Private Sub TrackAdded(ByVal source As Object, ByVal e As System.EventArgs)

    End Sub

    Private Sub TracksCleared(ByVal source As Object, ByVal e As System.EventArgs)
        '// clear the tangents list when the points are cleared
        '//tangentList.Clear();
    End Sub

#End Region
End Class
'End Namespace
