﻿'/*
'Axiom Game Engine Library
'Copyright (C) 2003  Axiom Project Team

'The overall design, and a majority of the core engine and rendering code 
'contained within this library is a derivative of the open source Object Oriented 
'Graphics Engine OGRE, which can be found at http://ogre.sourceforge.net.  
'Many thanks to the OGRE team for maintaining such a high quality project.

'This library is free software; you can redistribute it and/or
'modify it under the terms of the GNU Lesser General Public
'License as published by the Free Software Foundation; either
'version 2.1 of the License, or (at your option) any later version.

'This library is distributed in the hope that it will be useful,
'but WITHOUT ANY WARRANTY; without even the implied warranty of
'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
'Lesser General Public License for more details.

'You should have received a copy of the GNU Lesser General Public
'License along with this library; if not, write to the Free Software
'Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
'*/

Imports System
'imports Axiom.Collections
'Imports Axiom.Core
'imports Axiom.Controllers

'Namespace Animation
' <summary>
'		Represents the state of an animation and the weight of it's influence. 
' </summary>
' <remarks>
'		Other classes can hold instances of this class to store the state of any animations
'		they are using.
'		This class implements the IControllerValue interface to enable automatic update of
'		animation state through controllers.
' </remarks>
Public Class AnimationState
    Implements IControllerValue, IComparable

#Region " Member variables"

    ' <summary>Name of this animation track.</summary>
    Protected animationName As String
    ' <summary></summary>
    Protected msngTime As Single
    ' <summary></summary>
    Protected msngLength As Single
    ' <summary></summary>
    Protected inverseLength As Single
    ' <summary></summary>
    Protected msngWeight As Single
    ' <summary></summary>
    Protected mblnIsEnabled As Boolean

#End Region

#Region " Constructors"
    ' <summary>
    '		
    ' </summary>
    ' <param name="animationName"></param>
    ' <param name="time"></param>
    ' <param name="length"></param>
    ' <param name="weight"></param>
    ' <param name="isEnabled"></param>
    Friend Sub New(ByVal animationName As String, ByVal time As Single, ByVal length As Single, ByVal weight As Single, ByVal isEnabled As Boolean)
        Me.animationName = animationName
        Me.time = time
        Me.weight = weight
        Me.isEnabled = isEnabled

        ' set using the Length property
        Me.length = length
    End Sub

    ' <summary>
    ' 
    ' </summary>
    ' <param name="animationName"></param>
    ' <param name="time"></param>
    ' <param name="length"></param>
    Friend Sub New(ByVal animationName As String, ByVal time As Single, ByVal length As Single)
        Me.animationName = animationName
        Me.time = time
        Me.length = length
        Me.weight = 1.0F
        Me.isEnabled = False
    End Sub

#End Region

#Region " Properties"

    ' <summary>
    '		Gets the name of the animation to which this state applies
    ' </summary>
    Public Property Name() As String
        Get
            Return animationName
        End Get
        Set(ByVal Value As String)
            animationName = Value
        End Set
    End Property

    ' <summary>
    '		Gets/Sets the time position for this animation.
    ' </summary>
    Public Property Time() As Single
        Get
            Return msngTime
        End Get
        Set(ByVal Value As Single)
            msngTime = Value
        End Set
    End Property

    ' <summary>
    '		Gets/Sets the total length of this animation (may be shorter than whole animation)
    ' </summary>
    Public Property Length() As Single
        Get
            Return msngLength
        End Get
        Set(ByVal Value As Single)
            msngLength = Value

            ' update the inverse length of the animation
            If msngLength <> 0 Then
                inverseLength = 1.0F / msngLength
            Else
                inverseLength = 0.0F
            End If
        End Set
    End Property

    ' <summary>
    ' Gets/Sets the weight (influence) of this animation
    ' </summary>
    Public Property Weight() As Single
        Get
            Return msngWeight
        End Get
        Set(ByVal Value As Single)
            msngWeight = Value
        End Set
    End Property

    ' <summary>
    '		Gets/Sets whether this animation is enabled or not.
    ' </summary>
    Public Property IsEnabled() As Boolean
        Get
            Return mblnIsEnabled
        End Get
        Set(ByVal Value As Boolean)
            mblnisEnabled = Value
        End Set
    End Property

#End Region

#Region " Public methods"

    ' <summary>
    '		Modifies the time position, adjusting for animation length.
    ' </summary>
    ' <param name="offset">Offset from the current time position.</param>
    Public Sub AddTime(ByVal offset As Single)
        ' TODO: Add MathUtil function for this?
        time = CType(Math.IEEERemainder(time + offset, length), Single)

        If time < 0 Then
            time += length
        End If
    End Sub

    ' <summary>
    '     Clones this instance of AnimationState.
    ' </summary>
    ' <returns>A copy of this AnimationState object.</returns>
    '//        public AnimationState Clone() {
    '//            AnimationState newState = new AnimationState();
    '//            CopyTo(newState);
    '//            return newState;
    '//        }

    ' <summary>
    '     Copies the details of this AnimationState instance to another instance.
    ' </summary>
    ' <param name="target">Target instance to copy our details to.</param>
    Public Sub CopyTo(ByVal target As AnimationState)
        target.time = time
        target.animationName = animationName
        target.inverseLength = inverseLength
        target.length = length
        target.time = time
        target.weight = weight
    End Sub

#End Region

#Region " Implementation of IControllerValue"

    ' <summary>
    '		Gets/Sets the value to be used in a ControllerFunction.
    ' </summary>
    Public Property Value() As Single
        Get
            Return time * inverseLength
        End Get
        Set(ByVal Value As Single)
            time = Value * length
        End Set
    End Property
#End Region

#Region " Object overloads"

    ' <summary>
    '    Override GetHashCode.
    ' </summary>
    ' <remarks>
    '    Done mainly to quash warnings, no real need for it.
    ' </remarks>
    ' <returns></returns>
    Public Overrides Function GetHashCode() As Integer
        Return animationName.GetHashCode()
    End Function

#End Region

#Region " IComparable Members"

    ' <summary>
    '    
    ' </summary>
    ' <param name="obj"></param>
    ' <returns>0 if they are the same, -1 otherwise.</returns>
    Public Function CompareTo(ByVal obj As Object) As Integer Implements IComparable.CompareTo
        Dim other As AnimationState = CType(obj, AnimationState)

        If animationName = other.animationName And _
           isEnabled = other.isEnabled And _
           time = other.time And _
           weight = other.weight And _
           length = other.length Then

            Return 0
        Else
            Return -1
        End If
    End Function


#End Region
End Class
'end namespace
