'/*
'Axiom Game Engine Library
'Copyright (C) 2003  Axiom Project Team

'The overall design, and a majority of the core engine and rendering code 
'contained within this library is a derivative of the open source Object Oriented 
'Graphics Engine OGRE, which can be found at http://ogre.sourceforge.net.  
'Many thanks to the OGRE team for maintaining such a high quality project.

'This library is free software; you can redistribute it and/or
'modify it under the terms of the GNU Lesser General Public
'License as published by the Free Software Foundation; either
'version 2.1 of the License, or (at your option) any later version.

'This library is distributed in the hope that it will be useful,
'but WITHOUT ANY WARRANTY; without even the implied warranty of
'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
'Lesser General Public License for more details.

'You should have received a copy of the GNU Lesser General Public
'License along with this library; if not, write to the Free Software
'Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
'*/

Imports System
Imports System.Diagnostics
'Imports Axiom.Collections
'Imports Axiom.Core

'Imports Axiom.MathLib

'Namespace Animation
' <summary>
'		A 'track' in an animation sequence, ie a sequence of keyframes which affect a
'		certain type of object that can be animated.
' </summary>
' <remarks>
'		This class is intended as a base for more complete classes which will actually
'		animate specific types of object, e.g. a bone in a skeleton to affect
'		skeletal animation. An animation will likely include multiple tracks each of which
'		can be made up of many KeyFrame instances. Note that the use of tracks allows each animable
'		object to have it's own number of keyframes, i.e. you do not have to have the
'		maximum number of keyframes for all animable objects just to cope with the most
'		animated one.
'		<p/>
'		Since the most common animable object is a Node, there are options in this class for associating
'		the track with a Node which will receive keyframe updates automatically when the 'apply' method
'		is called.
' </remarks>
Public Class AnimationTrack
#Region " Fields"
    ' <summary>
    '		Handle of this animation track.
    '	</summary>
    Protected handle As Short

    ' <summary>
    '		Animation that owns this track.
    '	</summary>
    Protected parent As Animation

    ' <summary>
    '		Target node to be animated.
    '	</summary>
    Protected target As Node

    ' <summary>
    '		Maximum keyframe time.
    '	</summary>
    Protected maxKeyFrameTime As Single

    ' <summary>
    '		Collection of key frames in this track.
    '	</summary>
    Protected keyFrameList As KeyFrameCollection = New KeyFrameCollection()

    ' <summary>
    '		Flag indicating we need to rebuild the splines next time.
    '	</summary>
    Protected isSplineRebuildNeeded As Boolean

    ' <summary>
    '		Spline for position interpolation.
    '	</summary>
    Protected positionSpline As PositionalSpline = New PositionalSpline()
    ' <summary>
    '		Spline for scale interpolation.
    '	</summary>
    Protected scaleSpline As PositionalSpline = New PositionalSpline()
    ' <summary>
    '		Spline for rotation interpolation.
    '	</summary>
    Protected rotationSpline As RotationalSpline = New RotationalSpline()
    ' <summary>
    '		Defines if rotation is done using shortest path
    ' </summary>
    Protected useShortestPath As Boolean

#End Region

#Region " Constructors"

    ' <summary>
    '		Internal constructor, to prevent direction instantiation.  Should be created
    '		via a call to the CreateTrack method of an Animation.
    ' </summary>
    Friend Sub New(ByVal parent As Animation)
        '  Me.new(parent) ', Nothing)
    End Sub

    ' <summary>
    '		Internal constructor, to prevent direction instantiation.  Should be created
    '		via a call to the CreateTrack method of an Animation.
    ' </summary>
    'Friend Sub New(ByVal parent As Animation, ByVal target As Node)
    '    Me.parent = parent
    '    Me.target = target

    '    maxKeyFrameTime = -1

    '    ' use shortest path rotation by default
    '    useShortestPath = True
    'End Sub

#End Region

#Region " Properties"

    ' <summary>
    '		The name of this animation track.
    ' </summary>
    Public Property Handle() As Short
        Get
            Return Handle
        End Get
        Set(ByVal Value As Short)
            handle = Value
        End Set
    End Property

    ' <summary>
    '		Collection of the KeyFrames present in this AnimationTrack.
    ' </summary>
    Public ReadOnly Property KeyFrames() As KeyFrameCollection
        Get
            Return keyFrameList
        End Get
    End Property

    ' <summary>
    '		Gets/Sets the target node that this track is associated with.
    ' </summary>
    Public Property TargetNode() As Node
        Get
            Return target
        End Get
        Set(ByVal Value As Node)
            target = Value
        End Set
    End Property

#End Region

#Region " Public methods"

    ' <summary>
    '		Creates a new KeyFrame and adds it to this animation at the given time index.
    ' </summary>
    ' <remarks>
    '		It is better to create KeyFrames in time order. Creating them out of order can result 
    '		in expensive reordering processing. Note that a KeyFrame at time index 0.0 is always created
    '		for you, so you don't need to create this one, just access it using KeyFrames[0];
    ' </remarks>
    ' <param name="time">Time within the animation at which this keyframe will lie.</param>
    ' <returns>A new KeyFrame.</returns>
    Public Function CreateKeyFrame(ByVal time As Single) As KeyFrame
        Dim keyFrame As KeyFrame = New KeyFrame(Me, time)

        If time > maxKeyFrameTime Or (time = 0 And keyFrameList.Count = 0) Then
            keyFrameList.Add(keyFrame)
            maxKeyFrameTime = time
        Else
            ' search for the correct place to insert the keyframe
            Dim i As Integer = 0
            Dim kf As KeyFrame = keyFrameList(i)

            While kf.Time < time And i <> keyFrameList.Count
                i = i + 1
            End While

            keyFrameList.Insert(i, kf)
        End If

        ' ensure a spline rebuild takes place
        OnKeyFrameDataChanged()

        Return keyFrame
    End Function


    ' <summary>
    '		Gets a KeyFrame object which contains the interpolated transforms at the time index specified.
    ' </summary>
    ' <remarks>
    '		The KeyFrame objects held by this class are transformation snapshots at 
    '		discrete points in time. Normally however, you want to interpolate between these
    '		keyframes to produce smooth movement, and this method allows you to do this easily.
    '		In animation terminology this is called 'tweening'. 
    ' </remarks>
    ' <param name="time">The time (in relation to the whole animation sequence).</param>
    ' <returns>
    '		A new keyframe object containing the interpolated transforms. Note that the
    '		position and scaling transforms are linearly interpolated (lerp), whilst the rotation is
    '		spherically linearly interpolated (slerp) for the most natural result.
    ' </returns>
    Public Function GetInterpolatedKeyFrame(ByVal time As Single) As KeyFrame
        ' note: this is an un-attached keyframe
        Dim result As KeyFrame = New KeyFrame(Nothing, time)

        Dim k1 As KeyFrame, k2 As KeyFrame
        Dim firstKeyIndex As System.UInt16

        Dim t As Single = GetKeyFramesAtTime(time, k1, k2, firstKeyIndex)

        If t = 0.0F Then
            ' just use k1
            result.Rotation = k1.Rotation
            result.Translate = k1.Translate
            result.Scale = k1.Scale
        Else
            ' interpolate by t
            Dim mode As InterpolationMode = parent.InterpolationMode

            Select Case mode
                Case InterpolationMode.Linear
                    ' linear interoplation
                    result.Rotation = Quaternion.Slerp(t, k1.Rotation, k2.Rotation, useShortestPath)
                    result.Translate = Vector3.Lerp(k1.Translate, k2.Translate, t) ' k1.Translate + ((k2.Translate - k1.Translate) * t)
                    result.Scale = Vector3.Lerp(k1.Scale, k2.Scale, t)  ' k1.Scale + ((k2.Scale - k1.Scale) * t)

                    Exit Function

                Case InterpolationMode.Spline
                    ' spline interpolation
                    If isSplineRebuildNeeded Then
                        BuildInterpolationSplines()
                    End If

                    result.Rotation = rotationSpline.Interpolate(firstKeyIndex, t, useShortestPath)
                    result.Translate = positionSpline.Interpolate(firstKeyIndex, t)
                    result.Scale = scaleSpline.Interpolate(firstKeyIndex, t)
                    Exit Function

            End Select
        End If

        ' return the resulting keyframe
        Return result
    End Function

    ' <summary>
    '		Applies an animation track at a certain position to the target node.
    ' </summary>
    ' <remarks>
    '		When a track has bee associated with a target node, you can eaisly apply the animation
    '		to the target by calling this method.
    ' </remarks>
    ' <param name="time">The time position in the animation to apply.</param>
    ' <param name="weight">The influence to give to this track, 1.0 for full influence, less to blend with
    '		other animations.</param>
    ' <param name="accumulate"></param>
    Public Sub Apply(ByVal time As Single, ByVal weight As Single, ByVal accumulate As Boolean)
        ' call ApplyToNode with our target node
        ApplyToNode(target, time, weight, accumulate)
    End Sub

    ' <summary>
    '		Overloaded Apply method.  
    ' </summary>
    ' <param name="time"></param>
    Public Sub Apply(ByVal time As Single)
        ' call overloaded method
        Apply(time, 1.0F, False)
    End Sub

    ' <summary>
    '		Same as the Apply method, but applies to a specified Node instead of it's associated node.
    ' </summary>
    ' <param name="node"></param>
    ' <param name="time"></param>
    ' <param name="weight"></param>
    ' <param name="accumulate"></param>
    Public Sub ApplyToNode(ByVal node As Node, ByVal time As Single, ByVal weight As Single, ByVal accumulate As Boolean)
        Dim keyFrame As KeyFrame = Me.GetInterpolatedKeyFrame(time)

        If accumulate Then
            ' add to existing. Weights are not relative, but treated as absolute multipliers for the animation
            Dim translate As Vector3 = Vector3.Multiply(keyFrame.Translate, weight)
            node.Translate(translate)

            ' interpolate between not rotation and full rotation, to point weight, so 0 = no rotate, and 1 = full rotation
            Dim rotate As Quaternion = Quaternion.Slerp(Quaternion.Identity, keyFrame.Rotation, weight)
            node.Rotate(rotate)

            ' TODO: not yet sure how to modify scale for cumulative animations
            Dim scale As Vector3 = keyFrame.Scale
            node.Scale(scale)
        Else
            ' apply using weighted transform method
            node.WeightedTransform(weight, keyFrame.Translate, keyFrame.Rotation, keyFrame.Scale)
        End If
    End Sub


    ' <summary>
    '		Gets the 2 KeyFrame objects which are active at the time given, and the blend value between them.
    ' </summary>
    ' <remarks>
    '		At any point in time  in an animation, there are either 1 or 2 keyframes which are 'active',
    '		1 if the time index is exactly on a keyframe, 2 at all other times i.e. the keyframe before
    '		and the keyframe after.
    ' </remarks>
    ' <param name="time">The time index in seconds.</param>
    ' <param name="keyFrame1">Receive the keyframe just before or at this time index.</param>
    ' <param name="keyFrame2">Receive the keyframe just after this time index.</param>
    ' <param name="firstKeyIndex">If supplied, will receive the index of the 'from' keyframe incase the caller needs it.</param>
    ' <returns>
    '		Parametric value indicating how far along the gap between the 2 keyframes the time
    '    value is, e.g. 0.0 for exactly at 1, 0.25 for a quarter etc. By definition the range of this 
    '    value is:  0.0 &lt;= returnValue &lt; 1.0 .
    '</returns>
    Public Function GetKeyFramesAtTime(ByVal time As Single, ByRef keyFrame1 As KeyFrame, ByRef keyFrame2 As KeyFrame, ByRef firstKeyIndex As System.UInt16) As Single
        Dim firstIndex As Short = -1
        Dim totalLength As Single = parent.Length

        ' wrap time
        While time > totalLength
            time -= totalLength
        End While

        Dim i As Integer = 0

        ' makes compiler happy so it wont complain about this var being unassigned
        keyFrame1 = Nothing

        ' find the last keyframe before or on current time
        For i = 0 To keyFrameList.Count - 1 ' Step  i + 1
            Dim keyFrame As KeyFrame = keyFrameList(i)

            ' kick out now if the current frames time is greater than the current time
            If keyFrame.Time > time Then
                Exit For
            End If

            keyFrame1 = keyFrame
            firstIndex += 1S
        Next

        ' trap case where there is no key before this time
        ' use the first key anyway and pretend it's time index 0
        If firstIndex = -1 Then
            keyFrame1 = keyFrameList(0)
            firstIndex += 1S
        End If

        ' fill index of the first key
        firstKeyIndex = CType(firstIndex, System.UInt16)

        ' parametric time
        ' t1 = time of previous keyframe
        ' t2 = time of next keyframe
        Dim t1 As Single, t2 As Single

        ' find first keyframe after the time
        ' if no next keyframe, wrap back to first
        ' TODO: Verify logic
        If firstIndex = (keyFrameList.Count - 1) Then
            keyFrame2 = keyFrameList(0)
            t2 = totalLength
        Else
            keyFrame2 = keyFrameList(firstIndex + 1)
            t2 = keyFrame2.Time
        End If

        t1 = keyFrame1.Time

        If t1 = t2 Then
            ' same keyframe
            Return 0.0F
        Else
            Return (time - t1) / (t2 - t1)
        End If
    End Function

    ' <summary>
    '		Removes all key frames from this animation track.
    ' </summary>
    Public Sub RemoveAllKeyFrames()
        keyFrameList.Clear()

        ' ensure a spline rebuild takes place
        OnKeyFrameDataChanged()
    End Sub

    ' <summary>
    '		Removes the keyframe at the specified index.
    ' </summary>
    ' <param name="index">Index of the keyframe to remove from this track.</param>
    Public Sub RemoveKeyFrame(ByVal index As Integer)
        Debug.Assert(index < keyFrameList.Count, "Index of of bounds when removing a key frame.")

        keyFrameList.Remove(index)

        ' ensure a spline rebuild takes place
        OnKeyFrameDataChanged()
    End Sub

#End Region

#Region " Protected/Internal methods"

    ' <summary>
    '		Called internally when keyframes belonging to this track are changed, in order to
    '		trigger a rebuild of the animation splines.
    ' </summary>
    Friend Sub OnKeyFrameDataChanged()
        isSplineRebuildNeeded = True
    End Sub

    ' <summary>Used to rebuild the internal interpolation splines for translations, rotations, and scaling.</summary>
    Protected Sub BuildInterpolationSplines()
        ' dont calculate on the fly, wait till the end when we do it manually
        positionSpline.AutoCalculate = False
        rotationSpline.AutoCalculate = False
        scaleSpline.AutoCalculate = False

        positionSpline.Clear()
        rotationSpline.Clear()
        scaleSpline.Clear()

        ' add spline control points for each keyframe in the list
        Dim i As Integer
        For i = 0 To keyFrameList.Count - 1 'Step i + 1
            Dim keyFrame As KeyFrame = keyFrameList(i)

            positionSpline.AddPoint(keyFrame.Translate)
            rotationSpline.AddPoint(keyFrame.Rotation)
            scaleSpline.AddPoint(keyFrame.Scale)
        Next

        ' recalculate all spline tangents now
        positionSpline.RecalculateTangents()
        rotationSpline.RecalculateTangents()
        scaleSpline.RecalculateTangents()

        isSplineRebuildNeeded = False
    End Sub


#End Region
End Class
'end namespace
